<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Matthias Toscanelli <m.toscanelli@code-source.ch>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

require_once(t3lib_extMgm::extPath('cdsrc_gallery') . 'lib/class.tx_cdsrcgallery_div.php');

$GLOBALS['LANG']->includeLLFile('EXT:cdsrc_gallery/mod1/locallang.xml');
//require_once(PATH_t3lib . 'class.t3lib_scbase.php');
$GLOBALS['BE_USER']->modAccess($MCONF, 1); // This checks permissions and exits if the users has no permission for entry.
// DEFAULT initialization of a module [END]

/**
 * Module 'Images' for the 'cdsrc_gallery' extension.
 *
 * @author	Matthias Toscanelli <m.toscanelli@code-source.ch>
 * @package	TYPO3
 * @subpackage	tx_cdsrcgallery
 */
class tx_cdsrcgallery_module1 extends t3lib_SCbase {

    protected $pageinfo;
    
    protected $backPath = '../../../../typo3/';

    /**
     * Initializes the module.
     *
     * @return void
     */
    public function init() {
        parent::init();

        $this->setFolderFromId();
    }

    /**
     * Adds items to the ->MOD_MENU array. Used for the function menu selector.
     *
     * @return	void
     */
    public function menuConfig() {
        $this->MOD_MENU = array(
            'function' => array(
                '1' => $GLOBALS['LANG']->getLL('informations'),
                '2' => $GLOBALS['LANG']->getLL('configuration'),
            )
        );
        parent::menuConfig();
    }

    /**
     * Main function of the module. Write the content to $this->content
     * If you chose "web" as main module, you will need to consider the $this->id parameter which will contain the uid-number of the page clicked in the page tree
     *
     * @return void
     */
    public function main() {
        // Access check!
        // The page will show only if there is a valid page and if this page may be viewed by the user
        $this->pageinfo = t3lib_BEfunc::readPageAccess($this->id, $this->perms_clause);
        $access = is_array($this->pageinfo) ? 1 : 0;

        // Initialize doc
        $this->doc = t3lib_div::makeInstance('template');
        $this->doc->setModuleTemplate(t3lib_extMgm::extPath('cdsrc_gallery') . 'mod1/mod_template.html');
        $this->doc->backPath = $GLOBALS['BACK_PATH'];
        $docHeaderButtons = $this->getButtons();

        if (($this->id && $access) || ($GLOBALS['BE_USER']->user['admin'] && !$this->id)) {

            // Draw the form
            $this->doc->form = '<form action="" method="post" enctype="multipart/form-data">';

            // JavaScript
            $this->doc->JScode = '
                <script language="javascript" type="text/javascript">
                    script_ended = 0;
                    function jumpToUrl(URL)    {
                        document.location = URL;
                    }
                </script>
            ';
            $this->doc->postCode = '
                <script language="javascript" type="text/javascript">
                    script_ended = 1;
                    if (top.fsMod) top.fsMod.recentIds["web"] = 0;
                </script>
            ';
            // Render content:
            $this->moduleContent();
        } else {
            // If no access or if ID == zero
            $docHeaderButtons['save'] = '';
            $this->content .= $this->doc->spacer(10);
        }

        // Compile document
        $markers['FUNC_MENU'] = t3lib_BEfunc::getFuncMenu(0, 'SET[function]', $this->MOD_SETTINGS['function'], $this->MOD_MENU['function']);
        $markers['CONTENT'] = $this->content;

        // Build the <body> for the module
        $this->content = $this->doc->startPage($GLOBALS['LANG']->getLL('title'));
        $this->content .= $this->doc->moduleBody($this->pageinfo, $docHeaderButtons, $markers);
        $this->content .= $this->doc->endPage();
        $this->content = $this->doc->insertStylesAndJS($this->content);
    }

    /**
     * Prints out the module HTML.
     *
     * @return void
     */
    public function printContent() {
        $this->content .= $this->doc->endPage();
        echo $this->content;
    }

    /**
     * Generates the module content.
     *
     * @return void
     */
    protected function moduleContent() {
        switch ((int) $this->MOD_SETTINGS['function']) {
            case 1:
                $content = '';
                if (is_dir($this->id)) {
                    if (!is_writable($this->id)) {
                        $content .= '<div style="color:red;font-size:1.2em;font-weight:bold;text-align:center;">' . $GLOBALS['LANG']->getLL('directory_must_be_writable') . '</div>';
                    } else {

                        $fields = tx_cdsrcgallery_div::getFields($this->id);
                        $this->saveInformations($fields);
                        $page = intval(t3lib_div::_GET('page'));
                        $quantity = 40;

                        $ressources = array();
                        $folders = t3lib_div::get_dirs($this->id);
                        foreach ($folders as $folder) {
                            $ressources[] = array($this->id . '/' . $folder, 0);
                        }
                        $files = t3lib_div::getFilesInDir($this->id, tx_cdsrcgallery_div::getFileExtensionList(), TRUE);
                        foreach ($files as $file) {
                            $ressources[] = array($file, 1);
                        }

                        // Render header
                        $content .= '<thead><tr class="t3-row-header">';
                        $content .= '<td class="col-icon" nowrap="nowrap"></td>';
                        $content .= '<td nowrap="nowrap">' . $GLOBALS['LANG']->getLL('fields.filename') . '</td>';
                        foreach ($fields as $field) {
                            $content .= '<td nowrap="nowrap">' . (strlen($ll = $GLOBALS['LANG']->getLL('fields.' . $field)) > 0 ? $ll : $field) . '</td>';
                        }
                        $content .= '</tr></thead>';

                        // Render footer
                        $content .= '<tfoot><tr><td colspan="' . (2 + count($fields)) . '" style="text-align:center;">';
                        $pages = ceil(count($ressources) / $quantity);
                        for ($i = 0; $i < $pages; $i++) {
                            $content .= '<a href="' . $this->MCONF['_'] . '&id=' . $this->id . '&page=' . $i . '" style="' . ($page === $i ? 'font-weight:bold;' : '') . 'padding:0px 2px;">' . ($i + 1) . '</a>';
                        }
                        $content .= '</td></tr></tfoot>';

                        // Render ressources
                        $content .= '<tbody>';
                        $_ressources = array_slice($ressources, $page * $quantity, $quantity, FALSE);
                        foreach ($_ressources as $ressource) {
                            $path = $ressource[0];
                            $relPath = substr($path, strlen(PATH_site));
                            $informations = tx_cdsrcgallery_div::getInformations($path, $fields);
                            $basename = basename($path);
                            if ($ressource[1]) {
                                if (class_exists("\\TYPO3\\CMS\\Backend\\Utility\\BackendUtility")) {
                                    $img = \TYPO3\CMS\Backend\Utility\BackendUtility::getThumbNail($this->backPath . 'thumbs.php', '../'.$relPath);
                                }else{
                                    $img = t3lib_BEfunc::getThumbNail($this->backPath . 'thumbs.php', $path);
                                }
                                $title = $basename . '<br />' . $img;
                                $icon = '<span class="t3-icon t3-icon-mimetypes t3-icon-mimetypes-media t3-icon-media-image" title="' . $basename . '">&nbsp;</span>';
                            } else {
                                $title = $basename;
                                $icon = '<span class="t3-icon t3-icon-apps t3-icon-apps-filetree t3-icon-filetree-folder-default" title="' . $basename . '">&nbsp;</span>';
                            }

                            $content .= '<tr class="file_list_normal">';
                            $content .= '<td class="col-icon" nowrap="nowrap">' . $icon . '</td>';
                            $content .= '<td nowrap="nowrap">' . $title . '</td>';
                            foreach ($informations as $key => $info) {
                                $content .= '<td nowrap="nowrap"><textarea row="2" cols="20" name="informations[' . $relPath . '][' . $key . ']" style="width:100%;">' . $info . '</textarea></td>';
                            }
                            $content .= '</tr>';
                        }
                        $content .= '</tbody>';

                        $content = '<table id="typo3-filelist" cellspacing="0" cellpadding="0">' . $content . '</table>';
                    }
                }
                $this->content .= $this->doc->section('', $content, 0, 1);
                break;
            case 2:
                if (is_dir($this->id)) {
                    if (!is_writable($this->id)) {
                        $content .= '<div style="color:red;font-size:1.2em;font-weight:bold;text-align:center;">' . $GLOBALS['LANG']->getLL('directory_must_be_writable') . '</div>';
                    } else {
                        $filename = $this->id . '/gconf.txt';
                        $fieldsFromPost = t3lib_div::_POST('fields');
                        if (strlen($fieldsFromPost) > 0) {
                            $fields = implode('|', array_unique(array_merge(array('name'), t3lib_div::trimExplode('|', $fieldsFromPost, TRUE))));
                            $res = t3lib_div::writeFile($filename, $fields);
                        } else {
                            $fields = implode('|', tx_cdsrcgallery_div::getFields($this->id, 0));
                        }
                        $content = '<label for="fields">' . $GLOBALS['LANG']->getLL('configuration.fields') . '</label><br/>' .
                                '<input type="text" size="120" id="fields" name="fields" value="' . $fields . '"/>';
                        $this->content .= $this->doc->section('', $content, 0, 1);
                    }
                }
                break;
        }
    }

    /**
     * Save informations for folders and images
     * 
     * @param array $fields: Configured fields
     * 
     * @return void
     */
    protected function saveInformations($fields) {
        $informations = t3lib_div::_POST('informations');
        if (is_array($informations)) {
            foreach ($informations as $relPath => $infos) {
                $path = PATH_site . $relPath;
                if (preg_match('#^' . PATH_site . 'fileadmin/.*$#', $path) && file_exists($path)) {
                    $fileInformations = array();
                    foreach ($fields as $field) {
                        $fileInformations[] = isset($infos[$field]) ? $infos[$field] : '';
                    }
                    t3lib_div::writeFile(PATH_site . $relPath . '.txt', implode('|', $fileInformations));
                }
            }
        }
    }

    protected function saveFields() {
        
    }

    protected function setFolderFromId() {
        $this->id = rtrim(t3lib_div::_GP('id'), '/');

        if (t3lib_div::compat_version("6.0")) {
            list($none, $id) = t3lib_div::trimExplode(':', $this->id);
            $this->id = PATH_site . 'fileadmin' . $id;
        }

        // Make sure that $this->id is in fileadmin for sercurity reason
        $this->id = realpath($this->id);
        if (!preg_match('#^' . PATH_site . 'fileadmin/.*$#', $this->id)) {
            $this->id = 0;
        }
    }

    /**
     * Creates the panel of buttons for submitting the form or otherwise perform operations.
     *
     * @return array All available buttons as an assoc.
     */
    protected function getButtons() {
        $buttons = array(
            'csh' => '',
            'shortcut' => '',
            'save' => ''
        );

        // CSH
        $buttons['csh'] = t3lib_BEfunc::cshItem('_MOD_web_func', '', $GLOBALS['BACK_PATH']);

        // SAVE button
        $buttons['save'] = '<input type="image" class="c-inputButton" name="submit" value="Update"' . t3lib_iconWorks::skinImg($GLOBALS['BACK_PATH'], 'gfx/savedok.gif', '') . ' title="' . $GLOBALS['LANG']->sL('LLL:EXT:lang/locallang_core.php:rm.saveDoc', 1) . '" />';

        // Shortcut
        if ($GLOBALS['BE_USER']->mayMakeShortcut()) {
            $buttons['shortcut'] = $this->doc->makeShortcutIcon('', 'function', $this->MCONF['name']);
        }

        return $buttons;
    }

}

if (defined('TYPO3_MODE') && isset($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/cdsrc_gallery/mod1/index.php'])) {
    include_once($GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/cdsrc_gallery/mod1/index.php']);
}




// Make instance:
/** @var $SOBE tx_cdsrcgallery_module1 */
$SOBE = t3lib_div::makeInstance('tx_cdsrcgallery_module1');
$SOBE->init();

// Include files?
foreach ($SOBE->include_once as $INC_FILE) {
    include_once($INC_FILE);
}

$SOBE->main();
$SOBE->printContent();
?>