<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Matthias Toscanelli <m.toscanelli@code-source.ch>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

require_once(PATH_tslib . 'class.tslib_pibase.php');
require_once(t3lib_extMgm::extPath('cdsrc_gallery') . 'lib/class.tx_cdsrcgallery_div.php');

/**
 * Plugin 'Biceps - Maths en ligne' for the 'cdsrc_gallery' extension.
 *
 * @author	Matthias Toscanelli <m.toscanelli@code-source.ch>
 * @package	TYPO3
 * @subpackage	tx_cdsrcgallery
 */
class tx_cdsrcgallery_pi1 extends tslib_pibase {

    var $prefixId = 'tx_cdsrcgallery_pi1';  // Same as class name
    var $scriptRelPath = 'pi1/class.tx_cdsrcgallery_pi1.php'; // Path to this script relative to the extension dir.
    var $extKey = 'cdsrc_gallery'; // The extension key.
    var $pi_checkCHash = true;
    var $subCObj = null;

    /**
     * The main method of the PlugIn
     *
     * @param	string		$content: The PlugIn content
     * @param	array		$conf: The PlugIn configuration
     * @return	string The content that is displayed on the website
     */
    function main($content, $conf) {
        global $LANG;
        $this->conf = $conf;
        $this->pi_loadLL();

        $this->pi_setPiVarDefaults();
        $this->pi_initPIflexForm();

        $this->subCObj = t3lib_div::makeInstance('tslib_cObj');

        $root = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'root', 'global');
        if (strlen($root) === 0) {
            $root = $this->subCObj->stdWrap($this->conf['root'], $this->conf['root.']); // Get root from TypoScript
        }
        $root = rtrim($root, '/'); // Remove ending slash

        $path = PATH_site . $root;

        $content = '';

        // Check if root is in fileadmin and exists
        if (preg_match('#^fileadmin/.*$#', $root) && is_dir($path)) {
            $this->subCObj->start(array());
            $content = $this->subCObj->stdWrap($this->renderFolder($path), $this->conf['stdWrap.']);
        }

        return $this->pi_wrapInBaseClass($content);
    }

    /**
     * Render the view of a folder
     * 
     * @param string    $path: Path to current folder
     * @param integer   $depth: Depth of current folder 
     * @param integer   $index: Index of current folder 
     * @return string
     */
    protected function renderFolder($path, $depth = 0, $index = 0, $parentindex = '') {
        $cfolders = '';
        $cfiles = '';
        $fields = tx_cdsrcgallery_div::getFields($path);
        $dirindex = strlen($parentindex) === 0 ? $index : ($parentindex . '_' . $index);

        // Render sub-folders
        $folders = t3lib_div::get_dirs($path);
        $folderIndex = 0;
        sort($folders);
        if ($this->conf['reverseOrder'] && (!is_array($this->conf['reverseOrder.']) || !$this->conf['reverseOrder.']['exceptFolder'])) {
            $folders = array_reverse($folders);
        }
        foreach ($folders as &$folder) {
            $folder = $path . '/' . $folder;
            $cfolders .= $this->renderFolder($folder, $depth + 1, $folderIndex++, $dirindex);
        }

        // Render files
        $files = t3lib_div::getFilesInDir($path, tx_cdsrcgallery_div::getFileExtensionList(), TRUE);
        $fileIndex = 0;
        sort($files);
        if ($this->conf['reverseOrder'] && (!is_array($this->conf['reverseOrder.']) || !$this->conf['reverseOrder.']['exceptFile'])) {
            $files = array_reverse($files);
        }
        foreach ($files as $file) {
            $cfiles .= $this->renderFile($file, $fields, $dirindex, $fileIndex++);
        }

        $this->subCObj->start($this->appendInformationsToArray($path, array(
                    'basename' => basename($path),
                    'path' => substr($path, strlen(PATH_site)),
                    'parentindex' => $parentindex,
                    'dirindex' => $dirindex,
                    'index' => $index,
                    'folders' => $cfolders,
                    'files' => $cfiles,
                    'depth' => $depth,
                    'image' => substr($this->getFolderImage($path), strlen(PATH_site)),
                    'ifolders' => implode(',', $folders),
                    'ifiles' => implode(',', $files)
                        ), $fields));
        return $this->subCObj->cObjGetSingle($this->conf['folder'], $this->conf['folder.']);
    }

    /**
     * Render the view of a file
     * 
     * @param string    $path: The path to current file
     * @param array     $fields: List of customable additional fields
     * @param string    $dirindex: Index of parent folder 
     * @param integer   $index: Index of current file 
     * @return string
     */
    protected function renderFile($path, $fields, $dirindex, $index) {
        $basename = basename($path);
        $parts = explode('.', $basename);
        $extension = array_pop($parts);
        $name = implode('.', $parts);
        $this->subCObj->start($this->appendInformationsToArray($path, array(
                    'dirname' => rtrim(dirname($path), '/'),
                    'dirindex' => $dirindex,
                    'index' => $index,
                    'basename' => basename($path),
                    'path' => substr($path, strlen(PATH_site)),
                    'name' => $name,
                    'extension' => $extension
                        ), $fields));
        return $this->subCObj->cObjGetSingle($this->conf['file'], $this->conf['file.']);
    }

    /**
     * Append information to cObj array
     * 
     * @param string    $path: The path to ressource (file or folder)
     * @param array     $array: The current cObj array
     * @param array     $fields: List of customable additional fields
     * @return array
     */
    protected function appendInformationsToArray($path, $array, $fields) {
        $informations = array_values(tx_cdsrcgallery_div::getInformations($path, $fields));
        for ($i = 0, $ni = count($informations); $i < $ni; $i++) {
            $array['field_' . $i] = $informations[$i];
        }
        return $array;
    }

    /**
     * Get a folder image from a sub gallery
     * 
     * @param string    $path: Path to current folder
     * @return string
     */
    protected function getFolderImage($path) {
        $fileArr = array();
        $fileArr = t3lib_div::getAllFilesAndFoldersInPath($fileArr, $path, tx_cdsrcgallery_div::getFileExtensionList(), FALSE);
        sort($fileArr);
        if ($this->conf['reverseOrder'] && (!is_array($this->conf['reverseOrder.']) || !$this->conf['reverseOrder.']['exceptFile'])) {
            $files = array_reverse($files);
        }
        if (count($fileArr) > 0) {
            switch ($this->conf['folderImage']) {
                case 'first':
                    return array_shift($fileArr);
                case 'last':
                    return array_pop($fileArr);
                case 'random':
                default:
                    return $fileArr[array_rand($fileArr)];
            }
        }
        return '';
    }

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cdsrc_gallery/pi1/class.tx_cdsrcgallery_pi1.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cdsrc_gallery/pi1/class.tx_cdsrcgallery_pi1.php']);
}
?>